package cn.zhangdeman.filter;


import cn.zhangdeman.consts.RecordField;
import cn.zhangdeman.context.Request;
import cn.zhangdeman.context.RuntimeContext;
import jakarta.servlet.FilterChain;
import jakarta.servlet.FilterConfig;
import jakarta.servlet.ServletException;
import jakarta.servlet.ServletInputStream;
import jakarta.servlet.annotation.WebFilter;
import jakarta.servlet.http.Cookie;
import jakarta.servlet.http.HttpFilter;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.core.annotation.Order;

import java.io.IOException;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;


// 初始化请求
@WebFilter("/*") // 所有接口请求均会触发次过滤器
@Order(Ordered.REQUEST_ID) // 足够小, 保证最先执行
public class RequestInitFilter extends HttpFilter {

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
    }

    @Override
    public void destroy() {
    }

    @Override
    public void doFilter(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, FilterChain filterChain) throws IOException, ServletException {
        RuntimeContext runtimeContext = new RuntimeContext(); // 运行时上下文数据
        runtimeContext.setHttpServletRequest(httpServletRequest);
        runtimeContext.setHttpServletResponse(httpServletResponse);
        setBaseInfo(httpServletRequest,httpServletResponse, runtimeContext); // 设置基础信息
        setRequestQuery(runtimeContext); // 设置请求query信息
        setRequestBody(runtimeContext); // 设置请求Body
        setRequestHeaderAndCookie(runtimeContext); // 填充请求信息: header + cookie
        setRequestId(runtimeContext); // 设置请求ID, 每次请求需要重新生成
        httpServletRequest.setAttribute(RecordField.RUNTIME_THREAD_CONTEXT, runtimeContext); // 记录到请求上下文中
        // 继续向后执行
        filterChain.doFilter(httpServletRequest, httpServletResponse);
        // System.out.println("init request do filter 请求执行完成");
    }

    // 设置基础信息
    private void setBaseInfo(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, RuntimeContext runtimeContext) {
        // request 信息
        Request request = new Request();
        request.setClientIp(httpServletRequest.getRemoteAddr()); // client ip
        request.setRequestMethod(httpServletRequest.getMethod()); // 请求类型
        request.setRequestContentType(httpServletRequest.getContentType()); // 请求类型
        request.setRequestUri(httpServletRequest.getRequestURI()); // 请求uri
        runtimeContext.setRequestInfo(request); // 请求信息
        // server 信息
        runtimeContext.setServerIp(httpServletRequest.getLocalAddr()); // 服务器IP
        runtimeContext.setServerHostname(httpServletRequest.getLocalName()); // 服务器名称
        runtimeContext.setStartTimeStamp(System.currentTimeMillis()); // 开始请求时间
    }

    // 设置请求Query
    private void setRequestQuery(RuntimeContext runtimeContext) {
        Map<String, String> query = new HashMap<>();
        Map<String, String[]> queryParamList = runtimeContext.getHttpServletRequest().getParameterMap(); // query参数列表
        for (Map.Entry<String, String[]> entity : queryParamList.entrySet()) {
            if (entity.getValue().length == 0) {
                // 参数值为空
                query.put(entity.getKey(), "");
            } else {
                // 多个参数值,  取第一个
                query.put(entity.getKey(), entity.getValue()[0]);
            }
        }
        runtimeContext.getRequestInfo().setRequestQuery(query);
    }

    // 设置请求Body
    private void setRequestBody(RuntimeContext runtimeContext) throws IOException {
        HttpServletRequest httpServletRequest = runtimeContext.getHttpServletRequest(); // 原始请求实例
        ServletInputStream servletInputStream = httpServletRequest.getInputStream();
        byte[] requestBody = servletInputStream.readAllBytes();
        runtimeContext.getRequestInfo().setRequestBodyRaw(requestBody); // 原始请求Body
        runtimeContext.getRequestInfo().setRequestBody(Arrays.toString(requestBody)); // 请求body
    }


    // 设置请求信息: header + cookie
    private void setRequestHeaderAndCookie(RuntimeContext runtimeContext) {
        HttpServletRequest httpServletRequest = runtimeContext.getHttpServletRequest(); // 原始请求实例

        runtimeContext.getRequestInfo().setRequestHeader(new HashMap<>()); // 请求header
        runtimeContext.getRequestInfo().setRequestCookie(new HashMap<>()); // 请求cookie
        // 全部请求header
        Enumeration<String> enumeration = httpServletRequest.getHeaderNames();
        while (enumeration.hasMoreElements()) {
            String name = enumeration.nextElement();
            if (name.equalsIgnoreCase("cookie")) {
                // cookie单独摘出来处理
                continue;
            }
            String value = httpServletRequest.getHeader(name);
            runtimeContext.getRequestInfo().getRequestHeader().put(name, value);
        }
        // 全部请求cookie
        Cookie[] cookieList = httpServletRequest.getCookies();
        for (Cookie cookie : cookieList) {
            runtimeContext.getRequestInfo().getRequestCookie().put(cookie.getName(), cookie.getValue());
        }
    }

    // 设置请求ID
    private void setRequestId(RuntimeContext runtimeContext) {
        String traceId = runtimeContext.getHttpServletRequest().getHeader("X-Front-Trace-Id");
        List<String> list = new ArrayList<>();
        Instant instant = Instant.ofEpochMilli(runtimeContext.getStartTimeStamp());
        ZonedDateTime zdt = instant.atZone(ZoneId.systemDefault());
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String formatted = zdt.format(formatter);
        list.add(runtimeContext.getServerIp()); // 服务器IP
        list.add(formatted); // 请求时间
        list.add(UUID.randomUUID().toString()); // 随机字符串
        String requestId = String.join("-", list); // 拼接成请求ID
        if (null == traceId || traceId.isEmpty()) {
            traceId = requestId;
        }
        runtimeContext.setRequestId(requestId);
        runtimeContext.setTraceId(traceId);
    }

}
